@@ -140,7 +140,7 @@ class UNet2D(nn.Module):
     """
     2D U-Net
 
-    :param optional in_channels: number of input channels
+    :param optional in_channels: number of input channels, should be odd
     :param optional coi: indices that correspond to the classes of interest
     :param optional feature_maps: number of initial feature maps
     :param optional levels: levels of the encoder
@@ -156,6 +156,7 @@ class UNet2D(nn.Module):
         super(UNet2D, self).__init__()
 
         self.in_channels = in_channels
+        self.c = in_channels // 2
         self.coi = coi
         self.out_channels = len(coi)
         self.feature_maps = feature_maps
@@ -219,13 +220,14 @@ class UNet2D(nn.Module):
 
             # perform augmentation and transform to appropriate type
             x, _, y, y_ = augment_samples(data, augmenter=augmenter)
-            while (1 - y_).sum() == 0:  # make sure labels are not lost in augmentation, otherwise augment new sample
+            while (1 - y_[:, self.c:self.c + 1,
+                       ...]).sum() == 0:  # make sure labels are not lost in augmentation, otherwise augment new sample
                 x, _, y, y_ = augment_samples(data, augmenter=augmenter)
-            y = y.round().long()
+            y = y[:, self.c:self.c + 1, ...].round().long()
             # clean labels if necessary (due to augmentations)
             if len(self.coi) > 2:
                 y = clean_labels(y, len(self.coi))
-            y_ = y_.bool()
+            y_ = y_[:, self.c:self.c + 1, ...].bool()
 
             # zero the gradient buffers
             self.zero_grad()
@@ -261,7 +263,7 @@ class UNet2D(nn.Module):
 
             # log images if necessary
             if write_images:
-                log_images_2d([x], ['train/' + s for s in ['x']], writer, epoch=epoch)
+                log_images_2d([x[:, self.c:self.c + 1, ...]], ['train/' + s for s in ['x']], writer, epoch=epoch)
                 y_pred = F.softmax(y_pred, dim=1)
                 for i, c in enumerate(self.coi):
                     if not i == 0:  # skip background class
@@ -293,17 +295,17 @@ class UNet2D(nn.Module):
         cnt = 0
 
         # test loss
-        y_preds = []
-        ys = []
-        ys_ = []
+        y_preds = np.zeros((self.out_channels, 0))
+        ys = np.zeros((0))
+        ys_ = np.zeros((0))
         for i, data in enumerate(loader):
 
             # get the inputs and transfer to suitable device
             x, y = tensor_to_device(data, device)
             y_ = get_unlabeled(y)
             x = x.float()
-            y = get_labels(y, coi=self.coi, dtype=int)
-            y_ = get_labels(y_, coi=[0, 255], dtype=bool)
+            y = get_labels(y, coi=self.coi, dtype=int)[:, self.c:self.c + 1, ...]
+            y_ = get_labels(y_, coi=[0, 255], dtype=bool)[:, self.c:self.c + 1, ...]
 
             # forward prop
             y_pred = self(x)
@@ -314,22 +316,19 @@ class UNet2D(nn.Module):
             cnt += 1
 
             for b in range(y_pred.size(0)):
-                y_preds.append(F.softmax(y_pred, dim=1).data.cpu().numpy()[b, ...])
-                ys.append(y[b, 0, ...].cpu().numpy())
-                ys_.append(y_[b, 0, ...].cpu().numpy())
+                y_preds = np.concatenate(
+                    (y_preds, F.softmax(y_pred, dim=1)[b, ...].view(y_pred.size(1), -1).data.cpu().numpy()), axis=1)
+                ys = np.concatenate((ys, y[b, 0, ...].flatten().cpu().numpy()))
+                ys_ = np.concatenate((ys_, y_[b, 0, ...].flatten().cpu().numpy()))
 
         # prep for metric computation
-        y_preds = np.asarray(y_preds)
-        ys = np.asarray(ys)
-        w = (1 - np.asarray(ys_)).astype(bool)
-        js = [jaccard((ys == i).astype(int), y_preds[:, i, ...], w=w) for i in range(1, len(self.coi))]
-        ams = [accuracy_metrics((ys == i).astype(int), y_preds[:, i, ...], w=w) for i in
-               range(1, len(self.coi))]
+        w = (1 - ys_).astype(bool)
+        js = [jaccard((ys == i).astype(int), y_preds[i, :], w=w) for i in range(1, len(self.coi))]
+        ams = [accuracy_metrics((ys == i).astype(int), y_preds[i, :], w=w) for i in range(1, len(self.coi))]
 
         # don't forget to compute the average and print it
         loss_avg = loss_cum / cnt
-        print('[%s] Epoch %5d - Average test loss: %.6f'
-              % (datetime.datetime.now(), epoch, loss_avg))
+        print('[%s] Epoch %5d - Average test loss: %.6f' % (datetime.datetime.now(), epoch, loss_avg))
 
         # log everything
         if writer is not None:
@@ -346,7 +345,7 @@ class UNet2D(nn.Module):
 
             # log images if necessary
             if write_images:
-                log_images_2d([x], ['test/' + s for s in ['x']], writer, epoch=epoch)
+                log_images_2d([x[:, self.c:self.c + 1, ...]], ['test/' + s for s in ['x']], writer, epoch=epoch)
                 y_pred = F.softmax(y_pred, dim=1)
                 for i, c in enumerate(self.coi):
                     if not i == 0:  # skip background class
@@ -615,7 +614,7 @@ class UNet3D(nn.Module):
 
             # filter out unlabeled pixels and include them in augmentation
             y_ = get_unlabeled(data[1], dtype=float)
-            data.insert(1, y)
+            data.append(y)
             data.append(y_)
 
             # perform augmentation and transform to appropriate type
@@ -623,6 +622,9 @@ class UNet3D(nn.Module):
             while (1 - y_).sum() == 0:  # make sure labels are not lost in augmentation, otherwise augment new sample
                 x, _, y, y_ = augment_samples(data, augmenter=augmenter)
             y = y.round().long()
+            # clean labels if necessary (due to augmentations)
+            if len(self.coi) > 2:
+                y = clean_labels(y, len(self.coi))
             y_ = y_.bool()
 
             # zero the gradient buffers
@@ -649,8 +651,7 @@ class UNet3D(nn.Module):
 
         # don't forget to compute the average and print it
         loss_avg = loss_cum / cnt
-        print('[%s] Epoch %5d - Average train loss: %.6f'
-              % (datetime.datetime.now(), epoch, loss_avg))
+        print('[%s] Epoch %5d - Average train loss: %.6f' % (datetime.datetime.now(), epoch, loss_avg))
 
         # log everything
         if writer is not None:
@@ -692,9 +693,9 @@ class UNet3D(nn.Module):
         cnt = 0
 
         # test loss
-        y_preds = []
-        ys = []
-        ys_ = []
+        y_preds = np.zeros((self.out_channels, 0))
+        ys = np.zeros((0))
+        ys_ = np.zeros((0))
         for i, data in enumerate(loader):
 
             # get the inputs and transfer to suitable device
@@ -702,7 +703,7 @@ class UNet3D(nn.Module):
             y_ = get_unlabeled(y)
             x = x.float()
             y = get_labels(y, coi=self.coi, dtype=int)
-            y_ = get_labels(y_, coi=[0, 255], dtype=int)
+            y_ = get_labels(y_, coi=[0, 255], dtype=bool)
 
             # forward prop
             y_pred = self(x)
@@ -713,22 +714,19 @@ class UNet3D(nn.Module):
             cnt += 1
 
             for b in range(y_pred.size(0)):
-                y_preds.append(F.softmax(y_pred, dim=1).data.cpu().numpy()[b, ...])
-                ys.append(y[b, 0, ...].cpu().numpy())
-                ys_.append(y_[b, 0, ...].cpu().numpy())
+                y_preds = np.concatenate(
+                    (y_preds, F.softmax(y_pred, dim=1)[b, ...].view(y_pred.size(1), -1).data.cpu().numpy()), axis=1)
+                ys = np.concatenate((ys, y[b, 0, ...].flatten().cpu().numpy()))
+                ys_ = np.concatenate((ys_, y_[b, 0, ...].flatten().cpu().numpy()))
 
         # prep for metric computation
-        y_preds = np.asarray(y_preds)
-        ys = np.asarray(ys)
-        w = (1 - np.asarray(ys_)).astype(bool)
-        js = [jaccard((ys == i).astype(int), y_preds[:, i, ...], w=w) for i in range(1, len(self.coi))]
-        ams = [accuracy_metrics((ys == i).astype(int), y_preds[:, i, ...], w=w) for i in
-               range(1, len(self.coi))]
+        w = (1 - ys_).astype(bool)
+        js = [jaccard((ys == i).astype(int), y_preds[i, :], w=w) for i in range(1, len(self.coi))]
+        ams = [accuracy_metrics((ys == i).astype(int), y_preds[i, :], w=w) for i in range(1, len(self.coi))]
 
         # don't forget to compute the average and print it
         loss_avg = loss_cum / cnt
-        print('[%s] Epoch %5d - Average test loss: %.6f'
-              % (datetime.datetime.now(), epoch, loss_avg))
+        print('[%s] Epoch %5d - Average test loss: %.6f' % (datetime.datetime.now(), epoch, loss_avg))
 
         # log everything
         if writer is not None:
@@ -800,7 +798,7 @@ class UNet3D(nn.Module):
             # test the model for one epoch is necessary
             if epoch % test_freq == 0:
                 j = self.test_epoch(loader=test_loader, loss_fn=loss_fn, epoch=epoch, writer=writer,
-                                            write_images=True, device=device)
+                                    write_images=True, device=device)
 
                 # and save model if higher segmentation performance was obtained
                 if j > j_max:
